import pandas as pd
import streamlit as st
from fake_useragent import UserAgent
from humanfriendly import parse_size
from qbittorrentapi import Client
from qbittorrentapi.exceptions import APIConnectionError, LoginFailed
from requests import packages
from torrents.corsaronero import CorsaroNero
from torrents.limetorrents import LimeTorrents
from torrents.rarbg import RarBG
from torrents.torlock import TorLock
from torrents.torrentdownloads import TorrentDownloads
from torrents.x1337 import X1337
from urllib3.exceptions import InsecureRequestWarning

packages.urllib3.disable_warnings(category=InsecureRequestWarning)
st.set_page_config(page_title="GoldPirate", page_icon="☠️", layout="wide")
fake_ua = UserAgent()
qbt = Client(
    st.secrets.qbittorrent.url,
    st.secrets.qbittorrent.port,
    st.secrets.qbittorrent.username,
    st.secrets.qbittorrent.password,
)
SITES = {
    "1337x": X1337(fake_ua.random),
    "LimeTorrents": LimeTorrents(fake_ua.random),
    "TorLock": TorLock(fake_ua.random),
    "TorrentDownloads": TorrentDownloads(fake_ua.random),
    "CorsaroNero": CorsaroNero(fake_ua.random),
    "RarBG": RarBG(fake_ua.random),
}


def sort_all(torrents, sort):
    sort_type = {"size": (parse_size, 5), "seed": (int, 6), "leech": (int, 7)}
    if sort not in sort_type:
        return torrents
    return sorted(
        torrents, key=lambda x: sort_type[sort][0](x[sort_type[sort][1]]), reverse=True
    )


@st.cache_data(show_spinner=False)
def scraping(_site, name, query, pages, sort):
    return _site.build_list(query, pages, sort)


def main():
    st.title("_Gold Pirate_ ☠️")
    # query
    query_form = st.form("query-form")
    query = query_form.text_input("Query", placeholder="search your query here")
    left_col, right_col = query_form.columns(2)
    pages_labels = {1: "Fast", 2: "Normal", 3: "Deep"}
    pages = left_col.select_slider(
        "Search type", pages_labels, 2, format_func=lambda x: pages_labels.get(x)
    )
    sort = right_col.selectbox(
        "Sort", ["seed", "age", "size", "leech"], format_func=lambda x: x.title()
    )
    if query_form.form_submit_button("Search 🕵🏻‍♂️", use_container_width=True):
        if not query:
            return query_form.warning("**Query** can't be empty!", icon="⚠️")
        st.cache_data.clear()
    if not query:
        return

    # searching
    torrents_list = list()
    with st.status(f"**Searching** torrents for `{query}`", expanded=True) as status:
        for i, (name, site) in enumerate(SITES.items()):
            try:
                st.write(f"Searching in **{name}**..")
                torrents = scraping(site, name, query, pages, sort)
                torrents_list.extend(torrents)
                st.write(f"Found **{len(torrents)}** torrent(s)!")
            except Exception as e:
                raise e
                st.write("Error..")
                continue
        if not torrents_list:
            return status.update(
                label="Search complete with **NO** results..",
                state="complete",
                expanded=False,
            )
        status.update(
            label=f"Search complete, found **{len(torrents_list)}** torrent(s)!",
            state="complete",
            expanded=False,
        )

    # table
    torrents_sorted = sort_all(torrents_list, sort)
    columns = [
        "Site",
        "Torrent",
        "Torrent Page",
        "Age",
        "Category",
        "Size",
        "Seed",
        "Leech",
    ]
    torrents_df = pd.DataFrame(torrents_sorted, columns=columns)
    torrents_df["Download?"] = False
    data = st.data_editor(
        torrents_df,
        column_config={
            "Torrent Page": st.column_config.LinkColumn(
                display_text="Open page", width="small"
            ),
        },
        disabled=columns,
        use_container_width=True,
        hide_index=True,
    )
    if st.button("Download ⬇️", use_container_width=True):
        if (torrents_to_download := data.loc[data["Download?"]]).empty:
            return
        st.session_state["torrents"] = {
            i: (torrent["Site"], torrent["Torrent"].strip(), torrent["Torrent Page"])
            for i, torrent in torrents_to_download.iterrows()
        }
    if not st.session_state.get("torrents"):
        return

    # check QBitTorrent
    try:
        qbt.auth_log_in()
    except LoginFailed:
        return st.error("Invalid **username** and/or **password**!", icon="❌")
    except APIConnectionError:
        url = f"{qbt.host}:{qbt.port}"
        return st.warning(f"No **QBitTorrent** instance found at _{url}_", icon="⚠️")

    # download
    st.subheader("_Downloads_", divider="gray")
    for i, (site, torrent, page) in st.session_state.torrents.copy().items():
        container = st.empty()
        download_form = container.form(f"download-qbit-{i}")
        download_form.info(torrent, icon="📎")
        left_col, right_col = download_form.columns((2, 1))
        torrent_name = left_col.text_input("Name", placeholder="Torrent rename")
        folders = {path: name for name, path in st.secrets.paths.items()}
        file_folder = right_col.selectbox(
            "Folder", folders, format_func=lambda x: folders.get(x)
        )
        if download_form.form_submit_button("Download 🔗", use_container_width=True):
            if not file_folder:
                download_form.warning("Select a **folder**", icon="⚠️")
                continue
            try:
                magnet = SITES.get(site).get_magnet_link(page)
                qbt.torrents_add(magnet, rename=torrent_name, save_path=file_folder)
                st.session_state.torrents.pop(i)
                container.empty()
                st.toast("✅ Sent to _QBitTorrent_ **succesfully**!")
            except Exception:
                st.toast("❌ _Ops!_ Something went **wrong**..")
                continue


if __name__ == "__main__":
    main()



from datetime import datetime
from re import search, sub

import requests
from bs4 import BeautifulSoup as bs
from requests.exceptions import ConnectTimeout


class RarBG:
    def __init__(self, user_agent):
        self.url = "https://rargb.to"
        self.search = "?search=##"
        self.delimiter = "%20"
        self.sort = "?search=##&order=@@&by=DESC"
        self.sort_type = {
            "age": "data",
            "size": "size",
            "seed": "seeders",
            "leech": "leechers",
        }
        self.page = "/search/@@/"
        self.user_agent = {"User-Agent": user_agent}

    def _search_torrents(self, query, pages, sort=None):
        query_delim = sub(r"\s", self.delimiter, query)
        url_attach = (
            sub("@@", self.sort_type[sort], sub("##", query_delim, self.sort))
            if sort in self.sort_type
            else sub("##", query_delim, self.search)
        )
        torrents_per_page = [
            bs(
                requests.get(
                    f"{self.url}{sub('@@', str(i + 1), self.page)}{url_attach}",
                    headers=self.user_agent,
                    timeout=3,
                    verify=False,
                ).text,
                "html.parser",
            ).findAll("td")[45:-3]
            for i in range(pages)
        ]
        return sum(torrents_per_page, start=[])

    def build_list(self, query, pages, sort=None):
        try:
            search_list = self._search_torrents(query, pages, sort)
        except ConnectTimeout:
            return []
        singles = [search_list[i : i + 8] for i in range(0, len(search_list), 8)]
        torrents = list()
        for torrent in singles:
            name = torrent[1].text.strip().split("\n")[0]
            link_page = torrent[1].find("a").get("href")
            category = torrent[2].text.split("/")[0]
            time, size, seed, leech = [t.text for t in torrent[3:7]]
            torrents.append(
                [
                    "RarBG",
                    name,
                    f"{self.url}{link_page}",
                    self._date_converter(time),
                    category.title(),
                    sub(r"\d+$", "", size),
                    int(sub(",", "", seed)),
                    int(sub(",", "", leech)),
                ]
            )
        return torrents

    def _date_converter(self, site_date):
        if m := search(r"(\d{4})-(\d{2})-(\d{2})", site_date):
            year, month, day = m.groups()
            return datetime(int(year), int(month), int(day)).strftime("%d.%m.%Y")
        return ""

    def get_magnet_link(self, torrent_page):
        soup = bs(
            requests.get(torrent_page, allow_redirects=True, verify=False).text,
            "html.parser",
        )
        try:
            return [
                a.get("href")
                for a in soup.findAll("a")
                if search("magnet", a.get("href") or "")
            ][0]
        except IndexError:
            return None



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="goldpirate",
    version="1.6.4",
    author="Moris Doratiotto",
    author_email="moris.doratiotto@gmail.com",
    description="A python module to download torrents",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/mortafix/GoldPirate",
    packages=setuptools.find_packages(),
    install_requires=[
        "halo >= 0.0.31",
        "requests >= 2.28.1",
        "beautifulsoup4 >= 4.11.1",
        "colorifix >= 2.0.4",
        "qbittorrent-api  >= 2022.8.38",
        "tabulate >= 0.8.10",
        "fake_useragent >= 1.4",
        "humanfriendly >= 10.0",
    ],
    classifiers=[
        "Programming Language :: Python :: 3.8",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Intended Audience :: Developers",
    ],
    python_requires=">=3.8",
    keywords=["torrent", "download"],
    package_data={
        "goldpirate": [
            "config.json",
            "torrents/corsaronero.py",
            "torrents/limetorrents.py",
            "torrents/torlock.py",
            "torrents/torrentdownload.py",
            "torrents/x1337.py",
            "torrents/rarbg.py",
        ]
    },
    entry_points={"console_scripts": ["goldpirate=goldpirate.goldpirate:main"]},
)
